// ************************************************************
// Persistence Of Vision Ray Tracer Scene Description File
// File name  : baubles.pov
// Version    : MegaPOV 1.2.1
// Description: Christmas image for the POV-Team
// Date       : Dec 2005
// Author     : Jaime Vives Piqueres
// ************************************************************
#version unofficial MegaPov 1.2;

// *** standard includes ***
#include "colors.inc"
#include "textures.inc"
#include "metals.inc"
#include "transforms.inc"

// *** control center ***
#declare use_garlands  =1;
#declare use_blur    =7*0;  // blur samples (0=off)
#declare use_radiosity =2;  // two-pass: first use 2 then 1
#declare rad_brightness=1;
global_settings{
 max_trace_level 6
 assumed_gamma 1.5
 #if (use_radiosity)
  #include "i_rad-HDRI.inc"
 #end
}
#default{texture{finish{ambient 0}}}


// *** HDRI lighting ***
sphere {
  0,1
  pigment { 
   image_map { 
    // use your own, or download it from http://www.debevec.org/Probes/
    hdr "kitchen_probe" once interpolate 2 map_type 7  
   } 
  }
  finish { ambient .5 diffuse 0 }
  scale <-1,1,1>*10e5
  rotate 40*y
  hollow
}


// *** TEXTURES ***
#declare p_metal=
Brass*.25+Gold*.25;
#declare t_metal=
texture{
 pigment{p_metal}
 #if (use_radiosity<2)
 finish{reflection {p_metal,1} diffuse (1-(p_metal.gray+((1-p_metal.gray)/2)))}
 #end
}
#macro t_bauble(s_bauble)
#if (s_bauble.gray=0)
#local s_bauble=rgb <rand(r_col),rand(r_col),rand(r_col)>;
#end
#if (use_radiosity<2)
#local rfi=.5;
#local t_bauble_paint=
texture{
 pigment{s_bauble}
 finish{reflection {s_bauble*rfi,s_bauble.gray*rfi} conserve_energy}
}
#else
#local t_bauble_paint=
texture{pigment{s_bauble}}
#end
#local t_b=
texture{
 material_map{
  gif "map6a" map_type 1
  texture{t_metal normal{bump_map{gif "map6an" map_type 1 bump_size -1}}}
  texture{t_bauble_paint normal{bump_map{gif "map6a" map_type 1 bump_size -1}}}
 }
}
#local dummy=0;
texture{t_b}
#end


// *** TABLE ***
#declare table=
plane{y,0 
 #if (use_radiosity<2)
 texture{
  pigment{image_map{png "paper1"} rotate 90*x scale 9}
 }
 #else
 pigment{White}
 #end
 translate -10*y
}
object{table}


// *** BAUBLES ***
#include "i_bauble-1.inc"
#declare r_col=seed(439);
#declare r_bauble=seed(22);
#declare all_baubles=
union{
object{bauble_1(t_metal,t_bauble(Red*.67)) scale 5 
 rotate 360*rand(r_bauble)*y
 rotate (-80+160*rand(r_bauble))*x
 rotate 360*rand(r_bauble)*y
 translate <0,5,0>
}
object{bauble_1(t_metal,t_bauble(0)) scale 5
 rotate 360*rand(r_bauble)*y
 rotate (-80+160*rand(r_bauble))*x
 rotate 360*rand(r_bauble)*y
 translate <0,5,10>
}
object{bauble_1(t_metal,t_bauble(Yellow*.75)) scale 5
 rotate 360*rand(r_bauble)*y
 rotate (-80+160*rand(r_bauble))*x
 rotate 360*rand(r_bauble)*y
 translate <10,5,0> rotate -30*y
}
object{bauble_1(t_metal,t_bauble(0)) scale 5
 rotate 360*rand(r_bauble)*y
 rotate (-80+160*rand(r_bauble))*x
 rotate 360*rand(r_bauble)*y
 translate <10,5,0> rotate 28*y
}
object{bauble_1(t_metal,t_bauble(0)) scale 5
 rotate 360*rand(r_bauble)*y
 rotate (-80+160*rand(r_bauble))*x
 rotate 360*rand(r_bauble)*y
 translate <10,5,0> rotate 90*y
}
object{bauble_1(t_metal,t_bauble(0)) scale 5
 rotate 360*rand(r_bauble)*y
 rotate (-80+160*rand(r_bauble))*x
 rotate 360*rand(r_bauble)*y
 translate <-10,5,0> rotate 26*y
}
object{bauble_1(t_metal,t_bauble(0)) scale 5
 rotate 360*rand(r_bauble)*y
 rotate (-80+160*rand(r_bauble))*x
 rotate 360*rand(r_bauble)*y
 translate <-10,5,0> rotate -31*y
}
object{bauble_1(t_metal,t_bauble(0)) scale 5
 rotate 360*rand(r_bauble)*y
 rotate (-80+160*rand(r_bauble))*x
 rotate 360*rand(r_bauble)*y
 translate <-18,5,0> rotate 0*y
}
object{bauble_1(t_metal,t_bauble(0)) scale 5
 rotate 360*rand(r_bauble)*y
 rotate (-80+160*rand(r_bauble))*x
 rotate 360*rand(r_bauble)*y
 translate <17.5,5,0> rotate 0*y
}
object{bauble_1(t_metal,t_bauble(0)) scale 5
 rotate 360*rand(r_bauble)*y
 rotate (-80+160*rand(r_bauble))*x
 rotate 360*rand(r_bauble)*y
 translate <17.6,5,0> rotate 60*y
}
object{bauble_1(t_metal,t_bauble(0)) scale 5
 rotate 360*rand(r_bauble)*y
 rotate (-80+160*rand(r_bauble))*x
 rotate 360*rand(r_bauble)*y
 translate <-17.5,5,0> rotate 60*y
}
object{bauble_1(t_metal,t_bauble(0)) scale 5
 rotate 360*rand(r_bauble)*y
 rotate (-80+160*rand(r_bauble))*x
 rotate 360*rand(r_bauble)*y
 translate <17.6,5,0> rotate -60*y
}
object{bauble_1(t_metal,t_bauble(0)) scale 5
 rotate 360*rand(r_bauble)*y
 rotate (-80+160*rand(r_bauble))*x
 rotate 360*rand(r_bauble)*y
 translate <-17.5,5,0> rotate -60*y
}
}
object{all_baubles}
object{all_baubles translate -10*y rotate 120*y translate 5*x}


// *** GARLAND ***
#if (use_garlands)
#declare t_garland=
texture{
 #if (use_radiosity<2)
 T_Chrome_3D finish{ambient 0}
 #else
 pigment{Gray}
 #end
}
#include "i_garland2.inc"
#declare r_garland=seed(944);
#macro garland(zstart)
#declare ngs=120; // # of garland sections (spline)
#declare a_gnorm=array[ngs];
#declare a_gpos=array[ngs];
#declare igs=0;
#declare gpz=0;
#declare gpath=<-25,0,zstart>;
#declare traced_object=
union{
 object{all_baubles}
 object{all_baubles translate -10*y rotate 120*y translate 5*x}
 object{table}
}
// taking points for the splines with trace()
#while (igs<ngs)
 #declare gpz=gpz+(-.1+.2*rand(r_garland));
 #declare gpath=gpath+<50/ngs,0,gpz>;
 #declare Norm = <0, 0, 0>; 
 #declare Start = <gpath.x, 40, gpath.z>; 
 #declare Inter= trace ( traced_object, Start, -y, Norm ); 
 #declare a_gnorm[igs]=Norm;
 #declare a_gpos[igs]=Inter;
 #declare igs=igs+1;
#end
// create the normals spline
#local g_thr=.5;
#declare igs=1;
#local sp_gnorm=
spline{cubic_spline
0,a_gnorm[0]
#while (igs<ngs)
 #if (a_gpos[igs].y>8)
 (igs/ngs),a_gnorm[igs]
 #else
 (igs/ngs),y
 #end
 #declare igs=igs+1;
#end
}
// create the positions spline
#declare igs=1;
#local sp_garland=
spline{cubic_spline
0,a_gpos[0]
#while (igs<ngs)
 #if (a_gpos[igs].y>8)
 (igs/ngs),a_gpos[igs]
 #else
 (igs/ngs),<a_gpos[igs].x,8,a_gpos[igs].z>
 #end
 #declare igs=igs+1;
#end
}
// place garland pieces along spline
#declare igp=0;
#declare ngp=240;
union{
#while (igp<ngp)
 #if (use_radiosity<2)
 object{garland_section 
 #else
 sphere{0,6.77*.25 texture{t_garland}
 #end
  scale <.3,.3,.15>
  rotate 90*y 
  rotate rand(r_garland)*360*x
  rotate (-12+24*rand(r_garland))*z 
  rotate (-12+24*rand(r_garland))*y
  Reorient_Trans(y,sp_gnorm(igp/ngp))
  translate sp_garland(igp/ngp)+.67*.5*y
 }
 #declare igp=igp+1;
#end
}
#end // macro
object{garland(-1)}
//object{garland(-8)}
#end


// **************
// *** camera ***
// **************
#declare cl=<3,75,-15>;
#declare la=<0,-5,0>;
camera{
 location cl
 direction 3.5*z
 look_at la
 #if (use_blur)
 focal_point la
 aperture pi
 blur_samples use_blur
 #end
}

// for tests
#if (use_radiosity=0)
light_source{0,White
 translate cl+<1,2,3>
}
#end
